# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.labs import Labs
from ...types.providers import Providers

GetUserHistoricalPullsIntrospectRequestProvider = typing.Union[Providers, Labs]
