# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_destination_preferences import EventDestinationPreferences
from .libre_config import LibreConfig


class TeamConfig(UniversalBaseModel):
    libreview: LibreConfig
    texts_enabled: typing.Optional[bool] = None
    push_historical_data: typing.Optional[bool] = None
    provider_raw_data: typing.Optional[bool] = None
    reject_duplicate_connection: typing.Optional[bool] = None
    sdk_per_device_activity_timeseries: typing.Optional[bool] = None
    partial_webhooks_enabled: typing.Optional[bool] = None
    minor_ordering_enabled: typing.Optional[bool] = None
    eds_preferences: typing.Optional[EventDestinationPreferences] = None
    event_type_prefixes: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
