# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .parsed_lab_report_data import ParsedLabReportData
from .parsing_job_status import ParsingJobStatus


class ParsingJob(UniversalBaseModel):
    job_id: str
    status: ParsingJobStatus
    data: typing.Optional[ParsedLabReportData] = None
    needs_human_review: bool
    is_reviewed: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
