# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ParsingJobStatus(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    UPLOAD_PENDING = "upload_pending"
    STARTED = "started"
    COMPLETED = "completed"
    FAILED = "failed"
    _UNKNOWN = "__PARSINGJOBSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ParsingJobStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        upload_pending: typing.Callable[[], T_Result],
        started: typing.Callable[[], T_Result],
        completed: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ParsingJobStatus.UPLOAD_PENDING:
            return upload_pending()
        if self is ParsingJobStatus.STARTED:
            return started()
        if self is ParsingJobStatus.COMPLETED:
            return completed()
        if self is ParsingJobStatus.FAILED:
            return failed()
        return _unknown_member(self._value_)
