# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TraceElements(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    CHROMIUM = "chromium"
    COPPER = "copper"
    IODINE = "iodine"
    MANGANESE = "manganese"
    MOLYBDENUM = "molybdenum"
    SELENIUM = "selenium"
    _UNKNOWN = "__TRACEELEMENTS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TraceElements":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        chromium: typing.Callable[[], T_Result],
        copper: typing.Callable[[], T_Result],
        iodine: typing.Callable[[], T_Result],
        manganese: typing.Callable[[], T_Result],
        molybdenum: typing.Callable[[], T_Result],
        selenium: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TraceElements.CHROMIUM:
            return chromium()
        if self is TraceElements.COPPER:
            return copper()
        if self is TraceElements.IODINE:
            return iodine()
        if self is TraceElements.MANGANESE:
            return manganese()
        if self is TraceElements.MOLYBDENUM:
            return molybdenum()
        if self is TraceElements.SELENIUM:
            return selenium()
        return _unknown_member(self._value_)
