# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .get_user_historical_pulls_introspect_request_provider import GetUserHistoricalPullsIntrospectRequestProvider

__all__ = ["GetUserHistoricalPullsIntrospectRequestProvider"]
